@@ -25,6 +25,7 @@
 #define OBJDB_API struct objdb_iface_ver0
 #include "cnxman-socket.h"
 #include "nodelist.h"
+#include "fnvhash.h"
 
 #define MAX_PATH_LEN PATH_MAX
 
@@ -49,6 +50,7 @@ static char *mcast_name;
 static char *cluster_name;
 static char error_reason[1024] = { '\0' };
 static hdb_handle_t cluster_parent_handle;
+static int use_hashed_cluster_id = 0;
 
 /*
  * Exports the interface for the service
@@ -292,7 +294,6 @@ static uint16_t generate_cluster_id(char *name)
 		value <<= 1;
 		value += name[i];
 	}
-	sprintf(error_reason, "Generated cluster id for '%s' is %d\n", name, value & 0xFFFF);
 	return value & 0xFFFF;
 }
 
@@ -700,6 +701,11 @@ static void add_cman_overrides(struct objdb_iface_ver0 *objdb)
 			objdb->object_key_create_typed(object_handle, "join",
 						       "60", strlen("60")+1, OBJDB_VALUETYPE_STRING);
 		}
+		/* Extend fail_to_recv_const, see bz#587078 */
+		if (objdb_get_string(objdb, object_handle, "fail_recv_const", &value)) {
+			objdb->object_key_create_typed(object_handle, "fail_recv_const",
+						       "2500", strlen("2500")+1, OBJDB_VALUETYPE_STRING);
+		}
 		/* consensus should be 2*token, see bz#544482*/
 		if (objdb_get_string(objdb, object_handle, "consensus", &value)) {
 		        unsigned int token;
@@ -916,8 +922,14 @@ static int set_noccs_defaults(struct objdb_iface_ver0 *objdb)
 		return -1;
 	}
 
-	if (!cluster_id)
-		cluster_id = generate_cluster_id(cluster_name);
+	if (!cluster_id) {
+	    if (use_hashed_cluster_id)
+	        cluster_id = fnv_hash(cluster_name);
+	    else
+	        cluster_id = generate_cluster_id(cluster_name);
+
+	    sprintf(error_reason, "Generated cluster id for '%s' is %d\n", cluster_name, cluster_id);
+	}
 
 	if (!nodename_env) {
 		int error;
@@ -1084,6 +1096,7 @@ static int get_cman_globals(struct objdb_iface_ver0 *objdb)
 {
 	hdb_handle_t object_handle;
 	hdb_handle_t find_handle;
+	char *use_hash;
 
 	objdb_get_string(objdb, cluster_parent_handle, "name", &cluster_name);
 
@@ -1096,11 +1109,23 @@ static int get_cman_globals(struct objdb_iface_ver0 *objdb)
 		if (!key_filename)
 			objdb_get_string(objdb, object_handle, "keyfile", &key_filename);
 
+		objdb_get_string(objdb, object_handle, "hash_cluster_id", &use_hash);
+		if (use_hash) {
+		    if (strncasecmp(use_hash, "yes", 3) == 0 || strncasecmp(use_hash, "on", 2) == 0)
+		        use_hashed_cluster_id = 1;
+		}
+
 		if (!cluster_id)
 			objdb_get_int(objdb, object_handle, "cluster_id", &cluster_id, 0);
 
-		if (!cluster_id)
-			cluster_id = generate_cluster_id(cluster_name);
+		if (!cluster_id) {
+		    if (use_hashed_cluster_id)
+		        cluster_id = fnv_hash(cluster_name);
+		    else
+		        cluster_id = generate_cluster_id(cluster_name);
+
+		    sprintf(error_reason, "Generated cluster id for '%s' is %d\n", cluster_name, cluster_id);
+		}
 	}
 	objdb->object_find_destroy(find_handle);
 	return 0;
@@ -1203,6 +1228,8 @@ static void setup_old_compat(struct objdb_iface_ver0 *objdb, hdb_handle_t cluste
 	hdb_handle_t gfs_handle;
 	char *value;
 
+	use_hashed_cluster_id = 0;
+
 	/* Set groupd to backwards compatibility mode */
 	groupd_handle = find_or_create_object(objdb, "group", cluster_handle);
 	if (objdb->object_key_get(groupd_handle, groupd_compat, strlen(groupd_compat),
